/*! file
	File:			HostPalm.c

	Description:	This is the host portion of the example.  OS and device specific code to get the input, 
					load graphics, draw specific graphics, save and load games, and so on belongs 
					in here for use by the game.
	
					This file also provides a TapWave look and feel to the game. This code runs like
					a TapWave app, and can override the game's wishes to provide a best handheld
					interpretation of the game.

					This file is in C like most Palm programs.  It interacts with game.cpp to show
					how to work with C++ programs.

	Author:			Roger Flores

	Copyright:		Copyright  2003 by Tapwave, Inc.

	Disclaimer:		IMPORTANT:  This Tapwave software is provided by Tapwave, Inc. ("Tapwave").  Your 
					use is subject to and governed by terms and conditions of the Software Development 
					Kit Agreement ("SDK Agreement") between you and Tapwave.  If you have not entered 
					into Tapwaves standard SDK Agreement with Tapwave, you have no right or license 
					to use, reproduce, modify, distribute or otherwise exploit this Tapwave software.  
					You may obtain a copy of Tapwaves standard SDK Agreement by calling 650-960-1817
					or visiting Tapwave at http://www.tapwave.com/developers/. 

	Change History (most recent first):
				
*/

/*!

Glossary

game cycle:  Everything needed to advance the game one step.  This includes
    taking input, moving all world objects, triggering events, collision 
    detection, and drawing.

offscreen buffer: A set of pixels not used by the screen.  The graphics 
    are composed there and when done, made visible by the screen, by 
    copying them, or sometimes by pointing the screen hardware to them.
    
landscape mode: The screen placed into 480x320 mode.  No status bar is
    visible.  The Palm coordinate system remains 240x160 (low density).
    
game menu: A UI system to control or configure the game.  The method used
    draws over game in the offscreen buffer.  An alternative is to use 
    the Palm Menu system, but if so, be careful not to draw the game 
    which would overwrite the menus.

Palm bitmaps: Bitmaps in a native Palm format.  Typically created from
    BMP files with pilrc or pasted into Constructor and stored in the
    app's PRC.  Bitmaps can have alternate color depths and resolutions and are 
    collectively called a bitmap family.  The most appropriate family
    member is drawn.  The Palm API usually uses the hardware acceleration 
    when possible.
*/
#include <PalmOS.h>
#include <PenInputMgr.h>
#include "HostPalm.h"
#include <math.h>
#include "osdepend.h"
#include "driver.h"
#include "mamePalm.h"
#include "twlib.h"
#include "PlayGame.h"
#include "driver_prefix.h"
#include <PceNativeCall.h>
#include "StarterRsc.h"
#include "utils.h"

extern void	*my_emulStateP;
extern Call68KFuncType	*my_call68KFuncP;
static void MameAppStop();


#pragma option align=4

enum {
  hf_pin_present = 1              // has VG
  // 2, 4, 8, 16 ...
};

//AK Testing speed without locking/unlocking every frame
UInt8 *ScreenDestP;
UInt32 ZSoundEnable = 0; //AK - Changed to hold actual sample rate
UInt32 ZRotateEnable = 0; //AK - 0 means landscape, 1 means portrait (rot90)
UInt8 ZScreenStretch = 0; //AK - 0 means no stretch, 1 means fit to screen
Boolean IsZodiac = false;
/* This is the input that we regularly poll for each game cycle.  More
could be added. */
//static Int16 ShowFirstRom = 0;

AppJoystickInput InputPool;
static Int16 ROMCount = 0;
KeyStructType KeyStruct[15];
char *ConfigKeysP[30];
char ConfigKeysText[30][40]= 
{ "Exit:             ",
"P1 Credit:        ",
"P1 Start:         ",
"Up:               ",
"Down:             ",
"Left:             ",
"Right:            ",
"Button1:          ",
"Button2:          ",
"Button3:          ",
"Button4:          ",
"Menu:             ",
"Menu Select:      ",
"Pause:            ",
"Reset:            "};


// This defines the game menu, which is shown while the game is paused.
// It is reached by pressing the Home button.  Pressing Home again 
// switches apps to Home.  Pushing the joystick down selects the game
// resume.
#define GameMenuResume  0
#define GameMenuExit    1
#define GameMenuCount   2

#define twCreatorID             'Tpwv'
#define twFtrCreator            twCreatorID
#define twFtrAPIVersion         0x0000


// The ARM compiler does not link strings in a Position Independent code model.
// A workaround is to fully allocate the storage.
char GameMenuText[GameMenuCount][8] = 
{
    "Resume",
    "Exit"
};

typedef struct
{
	UInt16 driverIndex;
	UInt8 moduleIndex;
	char driverName[9];
	char driverLongName[31];
} DriverListType;

DriverListType *DriverList;

StatusBarType StatusBar;

int MenuOption;

    //TwGfxSurfaceType * pictureBitmapP[ObjectBitmapCount];
    //! These are only used if customBlitting is true.  They cache some of the info on
    //! the pictures that will be used often.  Custom blitting means using 
    //! a custom blitter drawing to a frame buffer, versus using the standard
    //! API calls.
    Boolean customBlitting;
    Boolean isInputMasked;
    UInt16 soundAmp;            // default sound amplitude
    UInt32 screenWidth;
    UInt32 screenHeight;
    UInt32 screenPitch;
    UInt32 menuWidth;
    UInt32 menuHeight;
	// This is the offscreen buffer which is copied to the screen.
    UInt8 * screenBufferP;  // DOLATER - This is really a pointer to the pixels!
    //! These are used to get the input.  isInputMasked prevents them
    //! from being set or unset repeatedly.  The padding inserted is
	//! to achieve ARM alignment for these data structures.
	//! NOTE: replace with compiler options when CW9 is avaiable.
    AppJoystickInput joystickInput;
    Int32 *inputFormatP;
    //! Indicates which game menu item is selected.
    Int16 gameMenuSelected;
    //! Controls when the next cycle starts
    UInt32 cycleNextTime;
    UInt32 cycleDurationTime;


//! Set to true if TapWave support is detected.  This means that those APIs
//! can be used to widen the screen and read the joystick.
Boolean TapWaveSupport;

//! Array of input information the app is interested in.  Other values 
//! like buttons could also be included here.
//!
//! NOTE: This structure must be four bytes aligned because it is used by ARM based
//! API.




static Int32 GTimeOut = 00;
bool stream_is_playing = false;
AudioDataBlockType dataBlock;
Err err;
SndStreamRef Zstream;
UInt8 *screenBufferScaleP;

bool custom_blitting;

/*! \brief  Mask the keys to reduce keyDownEvents from being sent.

This app does not use the events but polls the keys instead.  This also
sets up the joystick for analog polling.

\see HostUnmaskKeys

*/





void
HostMaskKeys()
{
    if (!isInputMasked)
    {
        // Set the keys we poll to not generate events.  This saves cpu cycles
        // by avoiding the events flowing through the event loop which we will 
        // ignore anyways.
        KeySetMask(~keyBitsAll);

        /*if (TapWaveSupport)
        {
            // Open an input queue for the joystick.  The queue is polled before
            // advancing each game cycle.
            TwInputOpen(&NewinputQueueP, "twinput", "r");
            
            // The input format must be four byte aligned.  A 
            // simple way to do this for all compilers is to allocate it and
            // move the data into it.  Metrowerks compilers (>= version 9)
            // can use "#pragma options align=4".  Gcc compilers can use 
            // "static Int32 InputFormats[] __attribute__ ((aligned (4))) = {...};"
            inputFormatP = MemPtrNew(sizeof(InputFormats));
            MemMove(inputFormatP, InputFormats, sizeof(InputFormats));
            TwInputSetFormat(NewinputQueueP, inputFormatP, sizeof(InputFormats));
            
            // Activating the queue stops the joystick from generating
            // keyDownEvents.  If this is not done, then every direction
            // visited while polling will cause a corresponding keyDownEvent
            // in that direction, and if the polling duration exceeds 
            // the keyDown repeat duration, then a repeating keyDownEvent 
            // will be generated too.
            TwInputActivate(NewinputQueueP);
        }*/
        
        
        isInputMasked = true;
    }
    
    return;
}


/*! \brief  Unmask the keys so they can generate events again.

Also, closes the joystick so it can generate events again.

\see HostMaskKeys
*/
void
HostUnmaskKeys()
{
    if (isInputMasked)
    {
        // Set the keys we poll to not generate events.  This saves cpu cycles.
        KeySetMask(keyBitsAll);
        
        /*if (TapWaveSupport)
        {
        	TwInputDeactivate(NewinputQueueP);
            TwInputClose(NewinputQueueP);
            
            MemPtrFree(inputFormatP);
        }*/
        
        isInputMasked = false;
    }
}




/*! \brief Create an offscreen buffer of the requested size.

Any existing buffer is freed.  This buffer is also freed by AppStop
before the app exits.

\see HostOpenScreenBuffer
\see HostDrawObject
\see HostCloseScreenBuffer

*/

//AK List of pixel types -- Just to show what depth can be
//#define twGfxPixelFormatRGB565          1                       /* little endian */
//#define twGfxPixelFormatRGB565_LE       1                       /* little endian */
//#define twGfxPixelFormatRGB565_BE       2                       /* big endian */
//#define twGfxPixelFormat2bpp            3                       /* 2 bits per pixel (colormap) */
//#define twGfxPixelFormat4bpp            4                       /* 4 bits per pixel (colormap) */
//#define twGfxPixelFormat8bpp            5                       /* 8 bits per pixel (colormap) */

/*void * GfxMalloc(UInt32 size, TwGfxSurfaceType *MemorySurface)
{
    TwGfxSurfaceInfoType surfInfo;
    Err error;



    // Setup the screenBuffer.  Clear it for use.
    surfInfo.size = sizeof(surfInfo);
    surfInfo.width = size/4+1;
    surfInfo.height = 2;
    surfInfo.location = twGfxLocationAcceleratorMemoryNoBackingStore;
    surfInfo.pixelFormat = twGfxPixelFormatRGB565;
    error = TwGfxAllocSurface(GFX, &MemorySurface, &surfInfo);
    if (!error)
		return MemorySurface;
	else
		return NULL;

}*/

/*void * GfxLock(TwGfxSurfaceType *MemorySurface, void *MemoryP)
{
	TwGfxLockSurface(MemorySurface, (void**)&MemoryP);
	return MemoryP;

}


void GfxFree(TwGfxSurfaceType *MemorySurface)
{
     TwGfxFreeSurface(MemorySurface);


}*/

/*void
HostNewScreenBuffer(short int buffer_width,
    short int buffer_height)
{
    TwGfxSurfaceInfoType surfInfo;
    Err error;


    if (screenBufferH)
    {
        TwGfxFreeSurface(screenBufferH);
        screenBufferH = NULL;
    }

    // Setup the screenBuffer.  Clear it for use.
    surfInfo.size = sizeof(surfInfo);
    surfInfo.width = buffer_width;
    surfInfo.height = buffer_height;
    surfInfo.location = twGfxLocationAcceleratorMemory;
    surfInfo.pixelFormat = twGfxPixelFormatRGB565;
    error = TwGfxAllocSurface(GFX, &screenBufferH, &surfInfo);

    screenBufferP = NULL;
	//TwGfxLockSurface(GameApp.screenBufferH, (void**)&GameApp.screenBufferP);

}*/

/*! \brief	Set draws to occur at the screen buffer

\see HostNewScreenBuffer
\see HostDrawObject
\see HostDrawGameMenu
\see HostDrawBuffer
\see HostCloseScreenBuffer
*/
/*void
HostOpenScreenBuffer(void)
{
    ErrNonFatalDisplayIf(screenBufferH == NULL ||
        screenBufferP != NULL, "Open screen");

    // Set the draw window to the screenBuffer so that all the draw operations act on the
    // screenBuffer which we draw later.
        TwGfxLockSurface(screenBufferH, (void**)&screenBufferP);
}*/


/* \brief	Draw the offscreen buffer to the screen.

Used after all drawing is done and typically after the buffer is closed to
transfer the results to the screen.
*/
/*#define screenTopLeftXOffset		0
#define screenTopLeftYOffset		0
void
HostDrawBuffer(short int destX,
    short int destY,
    short int sourceX,
    short int sourceY,
    short int sourceWidth,
    short int sourceHeight)
{
    TwGfxRectType sourceBounds;
    TwGfxPointType destPoint;
	
	//DrawGauges(sourceX, sourceY);
    
    sourceBounds.x = sourceX;
    sourceBounds.y = sourceY;
    sourceBounds.w = sourceWidth;
    sourceBounds.h = sourceHeight;
    destPoint.x = screenTopLeftXOffset + destX;
    destPoint.y = screenTopLeftYOffset + destY;

    // Copy the screenBuffer.  Have it wait until the 
    // next VSync to copy cleanly visually, and to give
    // improve performance.
    TwGfxBitBlt(displaySurface, &destPoint,
                screenBufferH, &sourceBounds);
}*/

/*! \brief Get the width of the Palm bitmap.

The result is in the native coordinate system (480x320).
NOTE: It's probably worthwhile to simply cache this information
after allocating the bitmap.

\see HostDrawObject
*/

// Assertion Support
/*static void
assert_check(int error, const char* message)
{
    if (error) {
        ErrDisplayFileLineMsg("PalmOS", (UInt16) error, message);
    }
}*/

#define xassert(error) assert_check(error, #error);

/*static void showError(const char* aWhat, Err aError) 
{
    ErrDisplayFileLineMsg("oink", (UInt16) aError, aWhat);
}

static void showError(const char* aWhat, UInt32 aValue) 
{
    char chbuf[50];
    StrPrintF(chbuf, "%lx", aValue);
    ErrDisplayFileLineMsg(aWhat, (UInt16)0, chbuf);
}*/


/*TwGfxSurfaceType* LoadBitmap(Int16 aResID)
{
    TwGfxSurfaceType* surf = NULL;
    TwGfxSurfaceInfoType surfInfo;
	TwGfxPointType dp= {0, 0};

#ifdef __PALMOS_ARMLET__
    MemHandle resH = DmGetResource('Tbmp', aResID);
#else
    MemHandle resH = DmGetResource('abmp', aResID);
#endif
    if (resH != NULL) {
        BitmapType* resBmp = (BitmapType*) MemHandleLock(resH);
        if (resBmp) {
            UInt16 prevcs = WinSetCoordinateSystem(kCoordinatesNative);

            // Get information about bitmap
            Coord w, h;
            WinGetBitmapDimensions(resBmp, &w, &h);

            // Allocate a surface to hold the image
            surfInfo.size = sizeof(surfInfo);
            surfInfo.width = w;
            surfInfo.height = h;
            surfInfo.location = twGfxLocationAcceleratorMemory;
            surfInfo.pixelFormat = twGfxPixelFormatRGB565;
            xassert(TwGfxAllocSurface(GFX, &surf, &surfInfo));

            xassert(TwGfxDrawPalmBitmap(surf, &dp, resBmp));
            xassert(MemHandleUnlock(resH));

            WinSetCoordinateSystem(prevcs);
        } else {
            assert_check(aResID, "MemHandleUnlock failed");
        }
        xassert(DmReleaseResource(resH));
    } else {
        assert_check(aResID, "DmGetResource failed");
    }

    return surf;
}*/

/*int
HostGetBitmapWidth(int bitmapNumber)
{
	TwGfxSurfaceInfoType surfInfo;
	
    surfInfo.size = sizeof(surfInfo);
    TwGfxGetSurfaceInfo(pictureBitmapP[bitmapNumber], &surfInfo);
    
    return surfInfo.width;
}*/


/*! \brief Get the height of the Palm bitmap.

The result is in the native coordinate system (480x320).
NOTE: It's probably worthwhile to simply cache this information
after allocating the bitmap.

\see HostDrawObject
*/

/*int
HostGetBitmapHeight(int bitmapNumber)
{
	TwGfxSurfaceInfoType surfInfo;
	
    surfInfo.size = sizeof(surfInfo);
    TwGfxGetSurfaceInfo(pictureBitmapP[bitmapNumber], &surfInfo);
    
    return surfInfo.height;
}*/

void HostFastDrawLive(UInt8 *buffer, int x, int y, int width, int height, UInt16 *pal16, int IAdjustBase)
{

	UInt8 *src;
	UInt16 *dest;
	int row,column;

	src = buffer;
#ifdef DEV_BUILD
  	HostOpenScreenBuffer();
#endif
  	dest = (UInt16 *)screenBufferP;
	if (dest)
	{
	// Draw bitmap quickly.
		//AK The +8 is to skip the safety area in the imagemap
		src+=(width+16)*8;
#ifndef DEV_BUILD
		dest += IAdjustBase;
#endif
		for (row = height; row > 0; row--)
		{
			src+=8;
			for (column = width; column > 0; column--)
			{
			  	*dest++ = pal16[*src++];
			}
			src+=8;
			dest += (screenWidth-width);
			
		}
	}
#ifdef DEV_BUILD
  HostCloseScreenBuffer();
#endif
}

/*! \brief	Draw an object at a specified location and mode

HostOpenScreenBuffer must be called first.

\see HostOpenScreenBuffer
\see HostDrawGameMenu
\see HostCloseScreenBuffer

*/


/*! \brief Draw the game menu visible while the game is paused.

Usually called by the game when it knows it is paused.  This function 
is in the Host side so it can use the Palm OS fonts and drawing API.

\see HostOpenScreenBuffer
\see HostDrawGameMenu
\see HostCloseScreenBuffer

*/


/*! \brief	Stop draws to the screen buffer

\see HostOpenScreenBuffer
\see HostNewScreenBuffer
\see HostDrawObject
\see HostDrawBuffer
*/
/*void
HostCloseScreenBuffer(void)
{
    // Now unlock the screen buffer surface. 
        TwGfxUnlockSurface(screenBufferH, true);    // DOLATER true needed?
        screenBufferP = NULL;
}*/


/* \brief  Return the time (ticks) to wait until the next game cycle.

\see AppHandleEvent
 */
static Int32
TimeUntillNextPeriod(void)
{
    Int32 timeRemaining = evtWaitForever;

	if (game_exists())
    {
        timeRemaining = (Int32) (cycleNextTime - TimGetTicks());
        if (timeRemaining < 0)
        {
            timeRemaining = 0;
        }
    }

    return timeRemaining;
}




/*! \brief The event loop for the aplication.  

The primary functions is to wait until the next game cycle and then
call the game to do it's thing.  But all other device input and activity 
is processed here too, like switching apps, turning off the device, 
alarms, and so on.
*/
/*static void
AppEventLoop(void)
{
    // Wait until the next game period.  This call either returns
    // immeadiately with an event, or blocks until the time requested
    // has passed, in which case it returns a nilEvent.
    // 
    // Palm OS does not have timers or threads or vertical syncs
    // available to signal a time to draw.  It only has machine ticks
    // which occur 1000 times per second while the device is awake.
    // 
    // To draw frames at a consistent interval we draw and then
    // perform work for the next frame that hopefully takes less time
    // than the desired frame rate permits.  If we finish early then
    // we wait the needed amount of ticks.  Otherwise we wait zero
    // ticks (but still allow the OS time for housekeeping).
    // 
    // An interesting topic is what to do when to much time is spent
    // preparing the next frame.  The two options are to do less work
    // or to slip a frame.
    // 
    // Doing less work can mean drawing less detail, less effects, or
    // lower quality drawing.  The idea is to not let the person
    // notice.  Applying lower quality to new visual elements can be
    // better than reducing something already seen.
    // 
    // Slipping a frame can be done occasionally without a noticeable
    // effect.  And when enough slippage happens it's the only way to
    // avoid a "rubberband" effect catching back up to the expected
    // time.  Skipping the drawing but not the world update is a good
    // way to recover some time but keep the world flowing smoothly.
    // It is a poor way to recover a lot of time because the player
    // feels out of control.
    // 
    // Not tracking the ticks, but simply waiting a constant time
    // makes the frame rate subject to the variability of the game
    // cycle.
    // 
    // It is important to qualify all game cycles for a typical game.
    // The game should have a mode to deterministically play through.
    // The number of ticks for each game cycle should be measured.
    // The average must be slightly less than the desired frame rate,
    // although a graph is much more informative.  The worst cases
    // should be recorded and replayed and examined to improve the
    // performance.  The worse game cycles are the most important,
    // because they cause the noticeable stuttering in otherwise silky
    // smooth games.
    //
    // The app stays in the loop above until an appStopEvent is
    // received by the user switching to another app.  Users of Palm
    // apps do not think of exiting an app, they simply go to another
    // app, and the current app gets an appStopEvent.

    //TwAppRun(&GTimeOut);
}*/


/*! \brief Load the graphic resources needed for the game.

Load the graphics into memory.  Custom blitting info is setup if needed too.  This
version uses gzipped BMP bitmaps.

*/
/*static void
AppLoadGraphics(App * app)
{
    int i;
    Err error;

    */
	// Keep the Object graphics locked because they are frequently used
    /* Bitmaps can be either Palm specific resources or custom resources.

       Palm specific resources are fine because there is tool support and 
       the blitting code is already written and hardware accelerated.  
       For an example using them, see the GoCart example.
       
       This code shows how to use a custom graphics format.  This example
       uses images in the BMP format because it supported by many tools.
       Since BMP images are not compressed when they have more than 256 
       colors, they are compressed using gzip on the desktop.  The
       images are listed in the RCP file and so included into the prc.
       
       Other formats could certainly be used, and might be perform better.
       So explore!
	*/
	
	/* This is a simple scheme that loads all graphics.  A smarter scheme 
	   would load the graphics only as they are needed.  This would reduce
	   memory usage by keeping less graphics around at any given point.  
	   This would also reduce load times by spreading the graphic loads
	   through out time.
	*/
    /*customBlitting = true;
    for (i = 0; i < ObjectBitmapCount; i++)
    {
        void * image;
        void * imageUncompressed;
        Int32 imageSize;
        MemHandle imageH;
        
        imageH = DmGetResource('BMPf', (UInt16) (ObjectBitmapFirst + i));
        image = MemHandleLock(imageH);
        imageSize = MemHandleSize(imageH);
        
        // Some of the images are compressed using gzip.  BMP images > 256 colors
        // are not compressed.  GZip is quite effective, quick and portable and makes a 
        // good choice.
        imageUncompressed = gunzip(image, imageSize);
        
        error = AppReadBMP(GFX, &pictureBitmapP[i], 
        	imageUncompressed == NULL ? image : imageUncompressed, imageSize);
        
        if (imageUncompressed != NULL)
        	MemPtrFree(imageUncompressed);
        
        MemHandleUnlock(imageH);
        DmReleaseResource(imageH);
        
        // If custom blitting can be used, then cache some of the info on
        // the pictures that will be used often. 
        if (customBlitting)
        {
            // Pixels in TwGfxSurfaces are not accessible until the surface is
            // locked.  Leave locked to avoid repeating the locking overhead,
            // but unlock later.
            error = TwGfxLockSurface(pictureBitmapP[i], &pictureBitmapBitsP[i]);
#if 0
            pictureIsTransparent[i] = BmpGetTransparentValue(
               pictureBitmapP[i], &pictureTransparencyIndex[i]);
#endif
        }
    }

}*/


// AK Add a ROM listing and selection menu so that we don't have to recompile
//	  every time we want a new game.  :)

void CountRoms()
{
	while(1)
	{
		if (drivers[ROMCount] == 0)
			break;
			
		++ROMCount;
	}


}


/*! \brief Obtain and configure resources needed to start the game.

Sets the screen to 16 bit color mode.

\see AppLoadGraphics

*/
UInt32 query_heap_free ( void ) {
  UInt16 heapid = 0;
  UInt32 max, bytes;

  MemHeapFreeBytes ( heapid, &bytes, &max );

  DEBUGS ( "heapfree" );
  DEBUGU32 ( bytes );

  return ( max );
}
Char* SafeStrCat ( Char *dst, const Char *src, UInt16 maxLengthWithNull, char *theFile, long theLine )
{
	UInt16   concatenated_length;
	//long   return_code;

	concatenated_length = StrLen ( dst ) + StrLen ( src );

	// First determine whether or not the concatenated length is greater than the maxLength

	if ( concatenated_length + 1 > maxLengthWithNull )
	{
		
		// Continue with the "safe" concatenation.
            // Note that StrNCat clips the destination string to n bytes including
            // the null character at the end, and it does append the null character.
		
		StrNCat ( dst, src, maxLengthWithNull );
		
	}  // if ( concatenated_length + 1 > maxLengthWithNull )

	// Just do the regular StrCat
	else
		StrCat ( dst, src );
		
	// return the destination
	return dst;
	
}  // SafeStrCat

Char* SafeStrCopy (Char* dst, const Char* src, UInt16 maxLengthWithNull, char *theFile, long theLine) 
{
	UInt16 srcLength;
	static long return_code;
	
	srcLength = StrLen(src);

	// First determine whether or not the source is greater than the maxLength

	if (srcLength + 1 > maxLengthWithNull )
	{
		
		// Continue with the "safe" copy
		
		StrNCopy ( dst, src, maxLengthWithNull - 1 );
		
		// Make sure we null terminate
		
		*(dst + maxLengthWithNull) = '\0';

	}  // if (srcLength + 1 > maxLengthWithNull )

	// Just do the regular StrCopy
	else
		StrCopy(dst, src);
		
	// return the destination
	return dst;
	
}  // SafeStrCopy
Char * SafeStrIToA ( Char *s, Int32 i, UInt16 maxLengthWithNull, char *theFile, long theLine )
{
      char   StrIToA_result [ maxStrIToALen ];
	UInt16   StrIToA_result_length;
	//long   return_code;
	
      // Determine what the actual destination string length WILL be.

	StrIToA ( StrIToA_result, i );

	StrIToA_result_length = StrLen ( StrIToA_result );

	// First determine whether or not the concatenated length is greater than the maxLength

	if ( StrIToA_result_length + 1 > maxLengthWithNull )
	{
		// Continue with the "safe" StrIToA.
		
		StrNCopy ( s, StrIToA_result, maxLengthWithNull - 1 );
		
		// Make sure we null terminate
		
		*(s + maxLengthWithNull) = '\0';
		
	}  // if ( StrIToA_result_length + 1 > maxLengthWithNull )

	// Just do the regular StrIToA 
	else
		StrIToA ( s, i );
		
	// return the destination
	return s;
	
}  // SafeStrIToA

/*SYSTEM_CALLBACK Boolean
AppHandleEvent(EventType * eventP)
{

}*/

typedef int (*NEW_STACK_F)(void * p);
int PNO_stack_switch(int stack_size, NEW_STACK_F func_p, void * data_p);
void * malloc(size_t size);
void free(void * memblock);
void profile_set_stack(void * sp, int len);
void profile_release_stack(void * sp, int len);

#pragma thumb off

typedef int (*NEW_STACK_F)(void * p);
int PNO_stack_switch(int stack_size, NEW_STACK_F func_p, void * data_p);
void * malloc(size_t size);
void free(void * memblock);
void profile_set_stack(void * sp, int len);
void profile_release_stack(void * sp, int len);

#pragma thumb off

asm int PNO_stack_switch(int stack_size, NEW_STACK_F func_p, void * data_p)
{
stmfd sp!,{r4-r8,lr}
mov r8,r0 // stack_size
mov r7,r1 // func_p
mov r6,r2 // data_p
add r0,r8,#7
bl malloc // malloc(stack_size + 7);
movs r4,r0 // store pointer in R4
moveq r0,#1 // return 1 if malloc() failed
ldmeqfd sp!,{r4-r8,lr}
bxeq lr

add r0,r8,#7
add r0,r4,r0
bic r5,r0,#7 // R5 = &p[stack_size + 7] & 0xFFFFFFF8

#ifdef BUILD_DEBUG
mov r0,r5
mov r1,r8
bl profile_set_stack
#endif

mov r0,r5 // switch stack
mov r5,sp // old stack pointer now in R5
mov sp,r0
mov r0,r6 // data_p
mov lr,pc
bx r7 // func_p
mov r6,r0 // store func_p result into R6

mov r0,r5 // restore stack
mov r5,sp
mov sp,r0

#ifdef BUILD_DEBUG
mov r0,r5
mov r1,r8
bl profile_release_stack
#endif

mov r0,r4
bl free // free pointer from R4
mov r0,r6 // return R6
ldmfd sp!,{r4-r8,lr}
bx lr
}

#pragma thumb reset


Err
MameAppStart()
{
    Err error;
    
    char temp[20];
    UInt16 prevcs;
    //TwGfxInfoType screenInfo;
    UInt32 featureValue;
	int MyKeycode=0;
	//TwGfxRectType aRect;
    UInt16 prefsSize;
    UInt16 prefsSize2;
    Int16 version;
    Int16 version2;
    Int16	GameIndex=-1;
    UInt16	TotalGames = 0;
    UInt16	count1=0;
    UInt16	CountDrivers = 0;
	UInt16 TempPrefs = 0;
	UInt32 *drawBitmapP;
	RectangleType screenRect;
	RGBColorType black = {0,0,0,0};
	char *TempData;
	int count;

	FtrGet('PDac',1112,&host_event_handler);

    isInputMasked = false;

    CountRoms();


	// Get GameState
	prefsSize = sizeof(Int16);
	version = PrefGetAppPreferences(appFileCreator, AppRunGameIndex, &GameIndex, &prefsSize, true);
	if (version == noPreferenceFound || prefsSize != sizeof(Int16))
	{
		//No prefs
		return errNone;

	}
	else
	{
		//Found prefs
		if (GameIndex == -1)
		{
			//AK -1 means list all the games to the preferences
		
			prefsSize2 = sizeof(UInt16);
			version2 = PrefGetAppPreferences(appFileCreator, AppModuleSizePref, &TotalGames, &prefsSize2, true);
			if (version2 == noPreferenceFound || prefsSize2 != sizeof(UInt16))
			{
				//No prefs
				TotalGames = 0;
				
				CountDrivers = ROMCount;
				TempPrefs = CountDrivers * sizeof(DriverListType);


				DriverList = (DriverListType *)malloc(TempPrefs);
				memset(DriverList,0,TempPrefs);

				for (count1 = 0;count1 < CountDrivers;count1++)
				{
					DriverList[count1].driverIndex = ByteSwap16(count1-TotalGames);
					DriverList[count1].moduleIndex = THIS_MODULE_NO;
					SafeStrCopy(DriverList[count1].driverName,drivers[count1]->name,9,__FILE__,__LINE__);
					SafeStrCopy(DriverList[count1].driverLongName,drivers[count1]->description,31,__FILE__,__LINE__);
				}
				TotalGames+=count1;
				TotalGames = ByteSwap16(TotalGames);
				
				PrefSetAppPreferences (appFileCreator, AppModuleSizePref, appPrefVersionNum, 
					&TotalGames, sizeof (UInt16), true);

				PrefSetAppPreferences (appFileCreator, AppDriverList, appPrefVersionNum, 
					DriverList,  TempPrefs, true);
				
				/*error = DmGetNextDatabaseByTypeCreator(true, &theSearchState, 'appl', appFileCreator, true, &theCardNo, &theDBID);
				
				// Launch the new app.
				if (!error)
					SysUIAppSwitch(theCardNo, theDBID, sysAppLaunchCmdNormalLaunch, NULL);
				*/

				//return errNone;
			}
			else
			{
				TotalGames = ByteSwap16(TotalGames);
				CountDrivers = ROMCount;
			
				//Prefs are there, load them for the list
				prefsSize = sizeof(DriverListType) * TotalGames;
				DriverList = (DriverListType *)malloc(sizeof(DriverListType) * (TotalGames + ROMCount));
				version = PrefGetAppPreferences(appFileCreator, AppDriverList, DriverList, &prefsSize, true);
				//Must read all drivers in first, then append the ones from this module
				

				//DriverList = (DriverListType *)malloc(TempPrefs);

				for (count1 = TotalGames;count1 < (CountDrivers+TotalGames);count1++)
				{
					DriverList[count1].driverIndex = ByteSwap16(count1-TotalGames);
					DriverList[count1].moduleIndex = THIS_MODULE_NO;
					SafeStrCopy(DriverList[count1].driverName,drivers[count1-TotalGames]->name,9,__FILE__,__LINE__);
					SafeStrCopy(DriverList[count1].driverLongName,drivers[count1-TotalGames]->description,31,__FILE__,__LINE__);
				}
				TotalGames+=ROMCount;
				TempPrefs = TotalGames * sizeof(DriverListType);

				TotalGames = ByteSwap16(TotalGames);
				
				PrefSetAppPreferences (appFileCreator, AppModuleSizePref, appPrefVersionNum, 
					&TotalGames, sizeof (UInt16), true);


				PrefSetAppPreferences (appFileCreator, AppDriverList, appPrefVersionNum, 
					DriverList,  TempPrefs, true);
			}
		}
		else
		{
			//AK GameIndex > -1 means run that game
			//Load Sound Prefs
			prefsSize = sizeof(UInt32);
			version = PrefGetAppPreferences(appFileCreator, AppSoundOption, &ZSoundEnable, &prefsSize, true);
			if (version == noPreferenceFound || prefsSize != sizeof(UInt32))
			{
				ZSoundEnable = 0;
			}
			//Rotate option
			prefsSize = sizeof(UInt32);
			version = PrefGetAppPreferences(appFileCreator, AppRotateOption, &ZRotateEnable, &prefsSize, true);
			if (version == noPreferenceFound || prefsSize != sizeof(UInt32))
			{
				ZRotateEnable = 0;
			}
			//Scaling option
			prefsSize = sizeof(UInt8);
			version = PrefGetAppPreferences(appFileCreator, AppScale, &ZScreenStretch, &prefsSize, true);
			if (version == noPreferenceFound || prefsSize != sizeof(UInt8))
			{
				ZScreenStretch = 0;
			}
			//Load Key Options
			prefsSize = sizeof(KeyStruct);
			version = PrefGetAppPreferences(appFileCreator, AppKeyOption, &KeyStruct, &prefsSize, true);
			if ( version == noPreferenceFound || prefsSize != sizeof(KeyStruct))
			{
				//Not Found
				MemSet(KeyStruct,sizeof(KeyStruct),0);
				
			}
			//StatusBar option
			prefsSize = sizeof(UInt8);
			version = PrefGetAppPreferences(appFileCreator, AppMenuOption, &MenuOption, &prefsSize, true);
			if (version == noPreferenceFound || prefsSize != sizeof(UInt8))
			{
				MenuOption = 0;
			}

		}
			
	
	}
	
	
  	//MemHeapCompact ( 0 );

	
	/*aRect.x = 0;
	aRect.y = 0;
	aRect.w = 480;
	aRect.h = 320;
 */

    IsZodiac = (Boolean) (FtrGet(twFtrCreator, twFtrAPIVersion, &featureValue) == errNone);
	TapWaveSupport = false;
#ifdef DEV_BUILD
	if (TapWaveSupport)
	{
		SysSetOrientation(sysOrientationLandscape);
		PINSetInputAreaState(pinInputAreaHide);
		StatHide();
		//TwAppStart(AppHandleEvent);
		screenInfo.size = sizeof(screenInfo);
    	error = TwGfxOpen(&GFX, &screenInfo);
    	if (error) return error;

    	screenWidth = screenInfo.displayWidth;
    	screenHeight = screenInfo.displayHeight;
    
    	error = TwGfxGetDisplaySurface(GFX, &displaySurface);
    	FontInit(GFX);
		TwAppStart(AppHandleEvent);
    	screenBufferH = NULL;
	    HostNewScreenBuffer((Coord) (512),
	        (Coord) (512));
	    screenWidth = 512;
	    screenHeight = 512;

    }
#else    
	FtrGet('PDac',1111,&drawBitmapP);
	screenBufferP = (UInt8 *)(drawBitmapP);
	
	/*prevcs = WinSetCoordinateSystem(kCoordinatesNative);

	WinGetDisplayExtent(&screenWidth, &screenHeight);
	
	WinSetCoordinateSystem(prevcs);*/
	
	FtrGet('PDac',1113,&screenWidth);
	FtrGet('PDac',1114,&screenHeight);
	FtrGet('PDac',1115,&screenPitch);
	//hw_set_fullscreen(1);
	
	//AK Set screen to black
	//MemSet(screenBufferP,(screenWidth * screenHeight) * sizeof(UInt16), 0);
	screenRect.topLeft.x = 0;
	screenRect.topLeft.y = 0;
	screenRect.extent.x = screenWidth;
	screenRect.extent.y = screenHeight;
	
	if (ZScreenStretch)
	{
		screenBufferScaleP = malloc(512 * 512 * sizeof(Int16));
		MemSet(screenBufferScaleP,512 * 512 * sizeof(Int16),0);
	}
	else
		screenBufferScaleP = NULL;
		
			
	WinSetForeColorRGB(&black,NULL);
	WinPaintRectangle(&screenRect,0);
#endif

	prevcs = WinSetCoordinateSystem(kCoordinatesNative);
	
	
	if (MenuOption == 2)//vertical
	{
		menuWidth = 24;
		menuHeight = 0;
	}
	else if (MenuOption == 1)//horiz
	{
		menuWidth = 0;
		menuHeight =24;
	}
	else//none
	{
		menuWidth = 0;
		menuHeight = 0;
	}
	
	if (MenuOption>0)
	{
		TempData = LoadImage("StatBar/Exit_Icon.bmp");
		if (TempData)
		{
			AppReadBMP(&StatusBar.StatIcons[0],TempData);
			MemPtrFree(TempData);
		}
		else
			error = 1;
		TempData = LoadImage("StatBar/Reset_Icon.bmp");
		if (TempData)
		{
			AppReadBMP(&StatusBar.StatIcons[1],TempData);
			MemPtrFree(TempData);
		}
		else
			error = 1;
		TempData = LoadImage("StatBar/P1Credit_Icon.bmp");
		if (TempData)
		{
			AppReadBMP(&StatusBar.StatIcons[2],TempData);
			MemPtrFree(TempData);
		}
		else
			error = 1;
		TempData = LoadImage("StatBar/P1Start_Icon.bmp");
		if (TempData)
		{
			AppReadBMP(&StatusBar.StatIcons[3],TempData);
			MemPtrFree(TempData);
		}
		else
			error = 1;
		TempData = LoadImage("StatBar/Menu_Icon.bmp");
		if (TempData)
		{
			AppReadBMP(&StatusBar.StatIcons[4],TempData);
			MemPtrFree(TempData);
		}
		else
			error = 1;
		TempData = LoadImage("StatBar/MenuSelect_Icon.bmp");
		if (TempData)
		{
			AppReadBMP(&StatusBar.StatIcons[5],TempData);
			MemPtrFree(TempData);
		}
		else
			error = 1;
		TempData = LoadImage("StatBar/IncFS_Icon.bmp");
		if (TempData)
		{
			AppReadBMP(&StatusBar.StatIcons[6],TempData);
			MemPtrFree(TempData);
		}
		else
			error = 1;
		TempData = LoadImage("StatBar/DecFS_Icon.bmp");
		if (TempData)
		{
			AppReadBMP(&StatusBar.StatIcons[7],TempData);
			MemPtrFree(TempData);
		}
		else
			error = 1;
		TempData = LoadImage("StatBar/Pause_Icon.bmp");
		if (TempData)
		{
			AppReadBMP(&StatusBar.StatIcons[8],TempData);
			MemPtrFree(TempData);
		}
		else
			error = 1;
		TempData = LoadImage("StatBar/VolUp_Icon.bmp");
		if (TempData)
		{
			AppReadBMP(&StatusBar.StatIcons[9],TempData);
			MemPtrFree(TempData);
		}
		else
			error = 1;
		TempData = LoadImage("StatBar/VolDown_Icon.bmp");
		if (TempData)
		{
			AppReadBMP(&StatusBar.StatIcons[10],TempData);
			MemPtrFree(TempData);
		}
		else
			error = 1;
		TempData = LoadImage("StatBar/VolMute_Icon.bmp");
		if (TempData)
		{
			AppReadBMP(&StatusBar.StatIcons[11],TempData);
			MemPtrFree(TempData);
		}
		else
			error = 1;
		TempData = LoadImage("StatBar/FPS_Icon.bmp");
		if (TempData)
		{
			AppReadBMP(&StatusBar.StatIcons[12],TempData);
			MemPtrFree(TempData);
		}
		else
			error = 1;
		TempData = LoadImage("StatBar/Bat1_Icon.bmp");
		if (TempData)
		{
			AppReadBMP(&StatusBar.StatIcons[13],TempData);
			MemPtrFree(TempData);
		}
		else
			error = 1;
		TempData = LoadImage("StatBar/Bat2_Icon.bmp");
		if (TempData)
		{
			AppReadBMP(&StatusBar.StatIcons[14],TempData);
			MemPtrFree(TempData);
		}
		else
			error = 1;
		TempData = LoadImage("StatBar/Bat3_Icon.bmp");
		if (TempData)
		{
			AppReadBMP(&StatusBar.StatIcons[15],TempData);
			MemPtrFree(TempData);
		}
		else
			error = 1;
	}
	
	if (error == 1)
		MenuOption = 0;

	if (MenuOption == 2)
	{
		for (count = 0;count<13;count++)
			RctSetRectangle(&StatusBar.RectStat[count],(screenWidth)-24,count*24,24,24);
			
			RctSetRectangle(&StatusBar.RectStat[count],(screenWidth)-24,count*24,24,8);
			RctSetRectangle(&StatusBar.RectStat[count+1],(screenWidth)-24,count*24,24,8);
			RctSetRectangle(&StatusBar.RectStat[count+2],(screenWidth)-24,count*24,24,8);
	}
	else if (MenuOption == 1)
	{
		for (count = 0;count<13;count++)
			RctSetRectangle(&StatusBar.RectStat[count],count*24,screenHeight-24,24,24);

			RctSetRectangle(&StatusBar.RectStat[count],count*24,screenHeight-24,8,24);
			RctSetRectangle(&StatusBar.RectStat[count+1],count*24,screenHeight-24,8,24);
			RctSetRectangle(&StatusBar.RectStat[count+2],count*24,screenHeight-24,8,24);

	}
	else
		MemSet(&StatusBar,sizeof(StatusBarType),0);

    // One goal of the game should be to display something as soon as possible.  If starting
    // the game or loading the graphics takes a noticeable time, it's probably good to display
    // a splash screen first.  This makes the app feel responsive.

    
    customBlitting = true;


	HostMaskKeys();




    // Now set up game info specific to this session.

    
    gameMenuSelected = 0;
    
    // Do a frame rate of 33 fps.
    cycleDurationTime = SysTicksPerSecond() / 33;
    
    
    //TwDeviceOpen(&GameApp.rumblerH, "vibrator0", "w");
    
    

    
    // Allocate a screen buffer one tile larger in both directions. 
    // Scrolling can cause part of the tiles on opposite ends to be 
    // visible at once.  Clipping is done when copying from the 
    // screen buffer to the screen.
    cycleNextTime = TimGetTicks() + cycleDurationTime;
    //query_heap_free();
    
    
			if (TapWaveSupport)
			{
#ifdef DEV_BUILD
				TwGfxFillRect(displaySurface, &aRect, TwGfxComponentsToPackedRGB(10,10,10));
    			TwGfxLockSurface ( displaySurface, (void**)&ScreenDestP );
#endif
    		}
    		if (GameIndex != -1)
    		{
    			MamePlayGame(GameIndex);
    		}
    		if (TapWaveSupport)
    		{
#ifdef DEV_BUILD
	        	TwGfxUnlockSurface ( displaySurface, true );
#endif
	        }
	        
	MameAppStop();
	
	
    return errNone;                 // no error
    
}


/*! \brief Cleanup the app by releasing and restoring all resources.

Frees the screen buffer.  Releases all graphics.  Restores the keys.
Closes all open forms.
*/
static void MameAppStop()
{
	EventType event;
	int i;
	
	for (i=0;i<16;i++)
		free(StatusBar.StatIcons->rowbytes);
	// Saving isn't implemented, but this is where it would be done.
	// Save the game automatically and restore to where the player left
	// off when the app returns.  The 90% case when starting the app
	// is to resume, using the Main menu is maybe the 10% case, so optimize
	// accordingly.
    //game_delete();
    
    //TwDeviceClose(GameApp.rumblerH);
    

    // Delete the screenBuffer
    /*if (screenBufferH)
    {
        TwGfxFreeSurface(screenBufferH);
        screenBufferH = NULL;
    }*/
	if (DriverList)
	{
		free(DriverList);
		DriverList = NULL;
	}
	if (ZScreenStretch)
	{
		free(screenBufferScaleP);
	}

    // Unlock and release the locked bitmaps
    /*for (i = 0; i < ObjectBitmapCount; i++)
    {
        TwGfxFreeSurface(app->pictureBitmapP[i]);
        app->pictureBitmapP[i] = 0;
    }
    */
    //GaugeClose(GameApp.SPFGauge);
    //GaugeClose(GameApp.FPSGauge);
    
    
    // Close the library. This frees up the surfaces we loaded too.
#ifdef DEV_BUILD
    TwGfxClose(GFX);
#endif
    //app->GFX = NULL;
    //app->displaySurface = NULL;
    
    //FtrUnregister('PDac',1111);
    //FtrUnregister('PDac',1112);
    //FtrUnregister('PDac',1113);
    //FtrUnregister('PDac',1114);
    //FtrUnregister('PDac',1115);
    
        // Restore the keys. 
    HostUnmaskKeys();
	EvtFlushPenQueue();

    
#ifdef DEV_BUILD
	TwAppStop();
#endif
	
}


static UInt32 asm ReplaceR13(UInt32 newValue)
{
    add r1, r0, #0    // save newValue in R1 scratch register
    add r0, r13, #0   // return original R13 value in R0
    add r13, r1, #0   // move new value into R13
    bx lr             // return to caller
}


    
